"""List of Odoo official addons."""

import sys
from enum import Enum
from functools import lru_cache
from typing import Set

if sys.version_info >= (3, 7):
    from importlib.resources import open_text
else:
    from importlib_resources import open_text

from ..odoo_series import OdooEdition, OdooSeries


@lru_cache()
def _get_core_addons(odoo_series: OdooSeries, odoo_edition: OdooEdition) -> Set[str]:
    with open_text(
        "manifestoo.core_addons", f"addons-{odoo_series.value}-{odoo_edition.value}.txt"
    ) as f:
        return {a.strip() for a in f if not a.startswith("#")}


@lru_cache()
def get_core_addons(odoo_series: OdooSeries) -> Set[str]:
    return _get_core_addons(odoo_series, OdooEdition.CE) | _get_core_addons(
        odoo_series, OdooEdition.EE
    )


def is_core_ce_addon(addon_name: str, odoo_series: OdooSeries) -> bool:
    return addon_name in _get_core_addons(odoo_series, OdooEdition.CE)


def is_core_ee_addon(addon_name: str, odoo_series: OdooSeries) -> bool:
    return addon_name in _get_core_addons(odoo_series, OdooEdition.EE)


def is_core_addon(addon_name: str, odoo_series: OdooSeries) -> bool:
    return addon_name in get_core_addons(odoo_series)


def get_core_addon_license(addon_name: str, odoo_series: OdooSeries) -> str:
    if is_core_ce_addon(addon_name, odoo_series):
        if odoo_series == OdooSeries.v8_0:
            return "AGPL-3"
        return "LGPL-3"
    elif is_core_ee_addon(addon_name, odoo_series):
        return "OEEL-1"
    raise AssertionError(f"{addon_name} is not a core addon.")  # pragma: no cover



import pytest

from manifestoo.addon import (
    Addon,
    AddonNotFoundInvalidManifest,
    AddonNotFoundNoManifest,
    AddonNotFoundNotADirectory,
    AddonNotFoundNotInstallable,
)


def test_basic(tmp_path):
    addon_dir = tmp_path / "theaddon"
    addon_dir.mkdir()
    (addon_dir / "__manifest__.py").write_text("{'name': 'the addon'}")
    addon = Addon.from_addon_dir(addon_dir)
    assert addon.name == "theaddon"
    assert addon.path == addon_dir
    assert addon.manifest.name == "the addon"


def test_not_a_directory(tmp_path):
    with pytest.raises(AddonNotFoundNotADirectory):
        Addon.from_addon_dir(tmp_path / "not-a-dir")


def test_not_installable(tmp_path):
    (tmp_path / "__manifest__.py").write_text("{'installable': False}")
    with pytest.raises(AddonNotFoundNotInstallable):
        Addon.from_addon_dir(tmp_path)
    assert Addon.from_addon_dir(tmp_path, allow_not_installable=True)


def test_invalid_manifest(tmp_path):
    (tmp_path / "__manifest__.py").write_text("[]")
    with pytest.raises(AddonNotFoundInvalidManifest):
        Addon.from_addon_dir(tmp_path)


def test_manifest_syntax_error(tmp_path):
    (tmp_path / "__manifest__.py").write_text("{'installable':}")
    with pytest.raises(AddonNotFoundInvalidManifest):
        Addon.from_addon_dir(tmp_path)


def test_manifest_type_error(tmp_path):
    (tmp_path / "__manifest__.py").write_text("{'installable': '?'}")
    with pytest.raises(AddonNotFoundInvalidManifest):
        Addon.from_addon_dir(tmp_path)


def test_no_manifest(tmp_path):
    with pytest.raises(AddonNotFoundNoManifest):
        Addon.from_addon_dir(tmp_path)



#!/usr/bin/env python
# setup.py generated by flit for tools that don't yet use PEP 517

from distutils.core import setup

packages = \
['manifestoo', 'manifestoo.commands', 'manifestoo.core_addons']

package_data = \
{'': ['*'],
 'manifestoo': ['.pytest_cache/*',
                '.pytest_cache/v/*',
                '.pytest_cache/v/cache/*'],
 'manifestoo.core_addons': ['.pytest_cache/*',
                            '.pytest_cache/v/*',
                            '.pytest_cache/v/cache/*']}

package_dir = \
{'': 'src'}

install_requires = \
['typer[all] ~=0.3.2']

extras_require = \
{":python_version<'3.7'": ['importlib_resources '],
 'doc': ['sphinx', 'furo', 'myst-parser', 'typer-cli', 'towncrier'],
 'test': ['pytest', 'coverage[toml]']}

entry_points = \
{'console_scripts': ['manifestoo = manifestoo.main:app',
                     'moo = manifestoo.main:app']}

setup(name='manifestoo',
      version='0.2',
      description='A tool to reason about Odoo addons manifests.',
      author='Stéphane Bidoul',
      author_email='stephane.bidoul@acsone.eu',
      url='https://github.com/sbidoul/manifestoo',
      packages=packages,
      package_data=package_data,
      package_dir=package_dir,
      install_requires=install_requires,
      extras_require=extras_require,
      entry_points=entry_points,
      python_requires='>=3.6',
     )



from typer.testing import CliRunner

from manifestoo.main import app
from manifestoo.version import __version__

runner = CliRunner()


def test_version():
    result = runner.invoke(app, ["--version"])
    assert result.exit_code == 0
    assert result.stdout == f"manifestoo version {__version__}\n"
